@@ -58,9 +58,14 @@ gboolean          gwget_application_open_window         (GwgetApplication   *app
 						      GError         **error);
 gboolean          gwget_application_open_uri            (GwgetApplication   *application,
 		                      const char      *uri,
-		                      const char      *destination_dir,
 						      guint32         timestamp,
 						      GError         **error);
+gboolean          gwget_application_open_uri_with_dest (GwgetApplication   *application,
+		                      const char      *uri,
+		                      const char      *destination_dir,
+						              guint32         timestamp,
+						              GError         **error);
+
 
 void	          gwget_application_open_uri_list       (GwgetApplication   *application,
 		  			              GSList          *uri_list,
