@@ -19,16 +19,16 @@
 #include <glade/glade.h>
 #include <gconf/gconf-client.h>
 #include <signal.h>
-#include <libgnome/gnome-url.h>
-#include <libgnome/gnome-program.h>
-#include <libgnome/gnome-init.h>
-#include <libgnomevfs/gnome-vfs.h>
+
 #include "main_window.h"
 #include "main_window_cb.h"
 #include "new_window.h"
 #include "gwget_data.h"
 #include "utils.h"
 #include "systray.h"
+#include "md5.h"
+
+#define MD5BUFSIZE 512
 
 static void inform_limit_speed_change(void);
 
@@ -51,21 +51,38 @@ on_treeview1_button_press_event(GtkWidget *widget, GdkEventButton *event,gpointe
 	GdkEventButton *event_button;
 	GtkTreeSelection *select;
 	GtkTreeIter iter;
-    GtkTreeModel *model;
+	GtkTreeModel *model;
 	GwgetData *gwgetdata;
 	gchar *uri;
-	GError *err = NULL; 
+	GError *err = NULL;
+	GFile *location; 
 	
 	treev=glade_xml_get_widget(xml,"treeview1");
 	select=gtk_tree_view_get_selection(GTK_TREE_VIEW(treev));
-		
+
+	/* Right click - Show popup menu */
 	if (event->type == GDK_BUTTON_PRESS) {
 		event_button = (GdkEventButton *) event;
 		if (event->button==3 && gtk_tree_selection_get_selected (select, &model, &iter)) {
-			gwget_data_set_menus(gwget_data_get_selected());
+			GtkTreeSelection *selection;
+			GtkTreePath *path;
+
+			/* Select right-clicked line */
+			selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(treev));
+			if (gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(treev),
+                                             (gint) event->x, 
+                                             (gint) event->y,
+                                             &path, NULL, NULL, NULL))
+			{
+				gtk_tree_selection_unselect_all(selection);
+				gtk_tree_selection_select_path(selection, path);
+				gtk_tree_path_free(path);
+			}
+
 			popup=glade_xml_get_widget(xml,"popup1");
 			gtk_menu_popup (GTK_MENU(popup), NULL, NULL, NULL, NULL, 
 							event_button->button, event_button->time);
+			gwget_data_set_popupmenu (gwget_data_get_selected());
 			return TRUE;
 		}
 	}
@@ -79,16 +96,19 @@ on_treeview1_button_press_event(GtkWidget *widget, GdkEventButton *event,gpointe
 			gwgetdata=gwget_data_get_selected();
 			/* If the download is in error state, show the error message in a dialog */
 			if (gwgetdata->error) {
-				run_dialog(_("Error in download"),gwgetdata->error_msg);
+				run_dialog_error(_("Error in download"),gwgetdata->error_msg);
 			} else {
 				if (gwgetdata->recursive) {
-					uri = gnome_vfs_make_uri_from_input(gwgetdata->dir);
+					location = g_file_parse_name (gwgetdata->dir); 
+					uri = g_file_get_uri (location);
+					g_object_unref (location);
 				} else {
-					uri = gnome_vfs_make_uri_from_input_with_dirs (gwgetdata->local_filename,
-                                                 GNOME_VFS_MAKE_URI_DIR_CURRENT);
+					location = g_file_new_for_commandline_arg (gwgetdata->local_filename);
+					uri = g_file_get_uri (location);
+					g_object_unref (location);
 				}
-				if (!gnome_url_show (uri, &err)) {
-					run_dialog(_("Error opening file"),_("Couldn't open the file"));
+				if (!gtk_show_uri (NULL, uri, GDK_CURRENT_TIME, &err)) {
+					run_dialog_error(_("Error opening file"),_("Couldn't open the file"));
 					return FALSE;
 				}
 				return TRUE;
@@ -157,7 +177,6 @@ on_about1_activate(GtkWidget *widget, gpointer data)
 
 	static const gchar *authors[] = { 
 			"David Sedeño Fernández <david@alderia.com>",
-			"Julien Cegarra <julienc@psychologie-fr.org>",
 			NULL
 	};
 		
@@ -168,7 +187,7 @@ on_about1_activate(GtkWidget *widget, gpointer data)
 	gtk_show_about_dialog (NULL,
 			       "name", _("Gwget"), 
 			       "version", VERSION,
-			       "copyright", "Copyright © 2004 David Sedeño Fernández",
+			       "copyright", "Copyright © 2004-2009 David Sedeño Fernández",
 			       "comments", _("Download Manager for GNOME."),
 			       "authors", authors,
 			       "translator-credits", _("translator-credits"),
@@ -195,7 +214,8 @@ new_download(GwgetData* gwgetdata)
 	GtkIconTheme *theme;
 	GtkIconInfo *icon_info;
 	GdkPixbuf *pixbuf;
-	char *mime, *size;
+	gchar *content_type = NULL;
+	gchar *size;
 	int width = 16, height = 16;
 	gdouble perc;
 	
@@ -223,9 +243,9 @@ new_download(GwgetData* gwgetdata)
 	
 	theme = gtk_icon_theme_get_default ();
 	if (!gwgetdata->recursive) {
-		mime = (gchar *)gnome_vfs_mime_type_from_name(gwgetdata->local_filename);
+		content_type = g_content_type_guess (gwgetdata->local_filename, NULL, 0, NULL);
 		gwgetdata->icon_name = gnome_icon_lookup (theme, NULL, NULL, NULL, NULL,
-	 							mime, GNOME_ICON_LOOKUP_FLAGS_NONE, NULL);
+	 							content_type, GNOME_ICON_LOOKUP_FLAGS_NONE, NULL);
 	} else {
 		gwgetdata->icon_name = g_strdup("gtk-refresh");
 	}
@@ -245,7 +265,8 @@ new_download(GwgetData* gwgetdata)
 
 	if (pixbuf)
 		g_object_unref (pixbuf);
-	
+		
+	g_free (content_type);
 }
 
 
@@ -253,7 +274,7 @@ void
 on_boton_pref_clicked(GtkWidget *widget, gpointer data)
 {
 	gchar *xml_file = NULL;
-	GtkWidget *window = NULL,*entry=NULL, *checkbutton=NULL;
+	GtkWidget *window = NULL,*entry=NULL, *checkbutton=NULL, *main_window = NULL;
 	
 	if (!xml_pref) {
 		xml_file=g_build_filename(DATADIR,"preferences.glade",NULL);
@@ -261,9 +282,12 @@ on_boton_pref_clicked(GtkWidget *widget, gpointer data)
 		glade_xml_signal_autoconnect(xml_pref);
 	}
 		
+	main_window = glade_xml_get_widget (xml, "main_window");
 	window = glade_xml_get_widget (xml_pref, "pref_window");
 	entry = glade_xml_get_widget(xml_pref,"save_in_entry");
 
+	gtk_window_set_transient_for(GTK_WINDOW(window), GTK_WINDOW(main_window));
+
 	if (gwget_pref.download_dir && strlen(gwget_pref.download_dir)!=0) {
 		gtk_entry_set_text(GTK_ENTRY(entry), gwget_pref.download_dir);
 	} else {
@@ -294,9 +318,9 @@ on_boton_pref_clicked(GtkWidget *widget, gpointer data)
 		gtk_entry_set_text(GTK_ENTRY(entry),gwget_pref.http_proxy);
 		checkbutton=glade_xml_get_widget(GLADE_XML(xml_pref),"proxy_uses_auth_radio");
 		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbutton),gwget_pref.proxy_uses_auth);
-		entry = glade_xml_get_widget(xml_pref,"proxy_user_entry");
+		entry = glade_xml_get_widget(xml_pref,"proxy_user");
 		gtk_entry_set_text(GTK_ENTRY(entry),gwget_pref.proxy_user);
-		entry = glade_xml_get_widget(xml_pref,"proxy_password_entry");
+		entry = glade_xml_get_widget(xml_pref,"proxy_password");
 		gtk_entry_set_text(GTK_ENTRY(entry),gwget_pref.proxy_password);
 
 	}
@@ -597,7 +621,7 @@ on_cancel_download_activate(GtkWidget *widget,gpointer data)
 	if (gwgetdata) 
 	{
 		msg = g_strdup_printf(_("Really cancel current download?\n(URL: %s)"),gwgetdata->url);
-		response = run_dialog(_("Cancel download?"),_(msg));
+		response = run_dialog(_("Delete download?"),_(msg), GTK_STOCK_CANCEL, _("Delete"));
 		if (response==GTK_RESPONSE_OK) {
 			gwget_data_stop_download(gwgetdata); 
 			unlink (gwgetdata->local_filename);
@@ -625,9 +649,9 @@ on_remove_completed_activate(GtkWidget *widget, gpointer data)
 	gint length,i;
 	gchar *url;
 	
-	if (g_list_length(downloads)>0) 
+	if (count_all_downloads()>0) 
 	{
-		response = run_dialog(_("Remove completed"),_("Really remove completed downloads from the list?"));
+		response = run_dialog(_("Remove completed"),_("Really remove completed downloads from the list?"), GTK_STOCK_CANCEL, _("Remove"));
 		if (response == GTK_RESPONSE_OK) {
 			length=gtk_tree_model_iter_n_children(GTK_TREE_MODEL(model),NULL);
 			gtk_tree_model_get_iter_root(model,&iter);
@@ -642,6 +666,8 @@ on_remove_completed_activate(GtkWidget *widget, gpointer data)
 					gtk_tree_model_iter_next(model,&iter);
 				}
 			}
+			gtk_widget_set_sensitive(glade_xml_get_widget(xml, "clear_button"), FALSE);
+			gtk_window_set_title(GTK_WINDOW(glade_xml_get_widget(xml, "main_window")), _("Gwget - Download Manager"));
 		}
 	}
 				
@@ -657,9 +683,9 @@ on_remove_notrunning_activate(GtkWidget *widget, gpointer data)
 	gint length,i;
 	gchar *url;
 	
-	if (g_list_length(downloads)>0) 
+	if (count_all_downloads()>0) 
 	{
-		response = run_dialog(_("Remove inactive"),_("Really remove inactive downloads from the list?"));
+		response = run_dialog(_("Remove inactive"),_("Really remove inactive downloads from the list?"), GTK_STOCK_CANCEL, _("Remove inactive"));
 		if (response == GTK_RESPONSE_OK) {
 			length=gtk_tree_model_iter_n_children(GTK_TREE_MODEL(model),NULL);
 			gtk_tree_model_get_iter_root(model,&iter);
@@ -689,9 +715,9 @@ on_remove_all_activate(GtkWidget *widget, gpointer data)
 	gint length,i;
 	gchar *url;
 	
-	if (g_list_length(downloads)>0) 
+	if (count_all_downloads()>0) 
 	{
-		response = run_dialog(_("Remove all"),_("Really remove all downloads from the list?"));
+		response = run_dialog(_("Remove all"),_("Really remove all downloads from the list?"), GTK_STOCK_CANCEL, _("Remove all"));
 		if (response == GTK_RESPONSE_OK) {
 			length=gtk_tree_model_iter_n_children(GTK_TREE_MODEL(model),NULL);
 			gtk_tree_model_get_iter_root(model,&iter);
@@ -721,7 +747,7 @@ on_view_toolbar_activate(GtkWidget *widget,gpointer data)
 	GtkWidget *toolbar,*menu_item;
 	gboolean state;
 	
-	toolbar=glade_xml_get_widget(GLADE_XML(xml),"bonobotoolbar");
+	toolbar=glade_xml_get_widget(GLADE_XML(xml),"toolbar1");
 	menu_item=glade_xml_get_widget(GLADE_XML(xml),"view_toolbar");
 	state = gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menu_item));
 	
@@ -771,10 +797,108 @@ on_properties_activate(GtkWidget *widget, gpointer data)
 		gtk_label_set_text(GTK_LABEL(local_file),gwgetdata->filename);
 		local_dir=glade_xml_get_widget(GLADE_XML(xml),"local_dir");
 		gtk_label_set_text(GTK_LABEL(local_dir),gwgetdata->dir);
-		gtk_widget_show(properties);
+		if (gwgetdata->state==DL_COMPLETED) {
+                   gtk_widget_set_sensitive(glade_xml_get_widget(xml, "compare_md5"), TRUE);
+		} else {
+                   gtk_widget_set_sensitive(glade_xml_get_widget(xml, "compare_md5"), FALSE);
+                }
+                gtk_widget_show(properties);
 	}
 }
 
+void
+on_compare_md5_clicked(GtkWidget *widget, gpointer data)
+{
+   GwgetData *gwgetdata;
+
+   gwgetdata=gwget_data_get_selected();
+	
+   if (gwgetdata) {
+      gtk_widget_show(glade_xml_get_widget(GLADE_XML(xml),"md5_window"));
+   }
+}
+
+void
+on_md5ok_button_clicked(GtkWidget *widget, gpointer data)
+{
+   GtkWidget *md5;
+   GwgetData *gwgetdata;
+   const gchar *entrytext;
+   GtkWidget *dialog;
+   GtkMessageType msg_type;
+
+   GString *md5res;
+   GString *msg;
+   gint i;
+   
+   struct md5_ctx md5context;
+   guchar md5digest[16];
+   gchar dataread[MD5BUFSIZE];
+   gboolean close_md5_dlg;
+   GFile *file;
+   GInputStream *stream;
+   gssize *sizeread;
+  
+   gwgetdata = gwget_data_get_selected();
+	
+   if (gwgetdata) {
+      md5=glade_xml_get_widget(GLADE_XML(xml),"md5_window");
+      
+      file = g_file_new_for_path (gwgetdata->local_filename);
+      
+      stream = (GInputStream *) g_file_read (file, NULL, NULL);
+      sizeread = (gssize *)g_input_stream_read (stream, dataread, MD5BUFSIZE, NULL,NULL);
+      
+      md5_init_ctx(&md5context);
+
+      while ( sizeread>0 ) {
+				md5_process_bytes (dataread, (int)sizeread, &md5context);
+				sizeread = (gssize *)g_input_stream_read (stream, dataread, MD5BUFSIZE, NULL,NULL);
+      }
+
+      entrytext=gtk_entry_get_text(GTK_ENTRY(glade_xml_get_widget(GLADE_XML(xml),"md5_entry")));
+
+      md5_finish_ctx(&md5context, md5digest);
+
+			md5res=g_string_new("");
+      for (i = 0; i < 16; i++) {
+         g_string_append_printf(md5res,"%02x", md5digest[i]);
+      }
+
+      msg = g_string_new("<span weight=\"bold\">");
+      
+      if (g_str_equal(md5res->str, entrytext)) {
+         close_md5_dlg=TRUE;
+         g_string_append_printf(msg, _("MD5SUM Check PASSED!"));
+         msg_type=GTK_MESSAGE_INFO;
+      } else {
+         g_string_append_printf(msg, _("MD5SUM Check FAILED!"));
+         close_md5_dlg=FALSE;
+         msg_type=GTK_MESSAGE_WARNING;
+      }
+      g_string_append_printf(msg, "</span>");
+
+      dialog = gtk_message_dialog_new_with_markup(GTK_WINDOW(md5),
+                                                     GTK_DIALOG_DESTROY_WITH_PARENT,
+                                                     msg_type,
+                                                     GTK_BUTTONS_CLOSE,
+                                                     NULL);
+
+      gtk_message_dialog_set_markup (GTK_MESSAGE_DIALOG(dialog), msg->str);
+                                  
+      gtk_dialog_run(GTK_DIALOG(dialog));
+      gtk_widget_destroy(GTK_WIDGET(dialog));   
+      g_string_free(md5res, TRUE);
+      g_string_free(msg, TRUE);
+      g_object_unref (stream);
+      g_object_unref (file);
+
+      if (close_md5_dlg) {
+         gtk_widget_hide(GTK_WIDGET(md5));
+      }
+   }
+}
+
 void 
 on_check_actual_size_toggled(GtkWidget *widget, gpointer data)
 {
@@ -1056,7 +1180,7 @@ on_download_menu_activate(void)
 void
 on_edit_menu_activate(void)
 {
-	if (g_list_length(downloads)>0) 
+	if (count_all_downloads()>0) 
 	{
 		 gtk_widget_set_sensitive(glade_xml_get_widget(xml, "remove_item"), TRUE);
 	} else {
@@ -1084,11 +1208,13 @@ on_remove_download_activate (GtkWidget *widget, gpointer data)
 	GwgetData *gwgetdata;
 	GtkTreeIter iter;
 	gchar *url;
+	gchar *message;
 	gint response, length, i;
 
 	gwgetdata = gwget_data_get_selected();
 	if (gwgetdata != NULL) {
-		response = run_dialog(_("Remove this download ?"),_("Really remove this download from the list?"));
+	    message = g_strdup_printf (_("Remove %s ?"), gwgetdata->filename);
+		response = run_dialog (message, _("Really remove this download from the list?"), GTK_STOCK_CANCEL, _("Remove download"));
 		if (response == GTK_RESPONSE_OK) {
 			length=gtk_tree_model_iter_n_children(GTK_TREE_MODEL(model),NULL);
 			gtk_tree_model_get_iter_root(model,&iter);
@@ -1110,18 +1236,22 @@ on_open_download_activate (GtkWidget *widget, gpointer data)
 {
 	GwgetData *gwgetdata;
 	gchar *uri;
+	GFile *location;
 	GError *err = NULL;
 
 	gwgetdata=gwget_data_get_selected();
 	g_return_if_fail (gwgetdata!=NULL);
 
-	uri = gnome_vfs_make_uri_from_input_with_dirs (gwgetdata->local_filename,
-							GNOME_VFS_MAKE_URI_DIR_CURRENT);
+	location = g_file_parse_name (gwgetdata->local_filename);
 
-	if (!gnome_url_show (uri, &err)) {
-		run_dialog(_("Error opening file"),_("Couldn't open the file"));
+	uri = g_file_get_uri (location);
+	
+	if (!gtk_show_uri (NULL, uri, GDK_CURRENT_TIME, &err)) {
+		run_dialog_error (_("Error opening file"),_("Couldn't open the file"));
 		return;
 	}
+
+	g_object_unref (location);
 }
 
 void
@@ -1129,18 +1259,22 @@ on_open_directory_activate (GtkWidget *widget, gpointer data)
 {
 	GwgetData *gwgetdata;
 	gchar *uri;
+	GFile *location;
 	GError *err = NULL;
 
 	gwgetdata=gwget_data_get_selected();
 	g_return_if_fail (gwgetdata!=NULL);
 
+	location = g_file_parse_name (gwgetdata->dir);
 
-	uri = gnome_vfs_make_uri_from_input(gwgetdata->dir);
+	uri = g_file_get_uri (location);
 
-	if (!gnome_url_show (uri, &err)) {
-		run_dialog(_("Error opening file"),_("Couldn't open the folder"));
+	if (!gtk_show_uri (NULL, uri, GDK_CURRENT_TIME, &err)) {
+		run_dialog_error (_("Error opening file"),_("Couldn't open the folder"));
 		return;
 	}
+
+	g_object_unref (location);
 }
 
 void
